function [N,R,Ch]=two_concentrations(M,L,bg)
%[N,R,Ch]=two_concentrations(M,L,bg) 
%
%Calculates concentration of two substances from the absorption spectrum of their mixture. 
%
%The first column of the matrix "M" is the absorption coefficient of the first substance, 
% the second column is the absorption coefficient of the second substance, 
% the third column is the measured spectrum (absorbance) of the mixture 
% and the optional fourth column are weights. 
%"L" is the absorption length. 
%If "bg"=1, the background is also fitted. If "bg"=0, the background is supposed to be zero. 
%
%The first two numbers in the vector "N" are concentrations of both substances. 
% the eventual third number in the vector "N" is the fitted background level, which is calculated for "bg"=1 only. 
%"R" are the standard widths of the confidence intervals for the fitted quantities (components of the vector "N"). 
%Some additional data are in the matrix "Ch": 
%The first line of the matrix "Ch" are the mean sum of squares of errors and the mean relative deviation. 
%The second line of the matrix "Ch" indicates how big part of the sum of squares is related to the first and to the second substance. 

[n,sloupce]=size(M); %M: The first column is the spectrum (absorption coef.) of the first substance, the second solumn is the spectrum of the second substance, the third column is the measured spectrum (absorbance), eventual fourth column are weights.  
if sloupce==3
 M(:,4)=ones(n,1); %Unit weights for all measured points 
elseif sloupce==4 
 M(:,4)=M(:,4)*n/sum(M(:,4)); %Normalization of weights (sum of all weights is "n") 
else 
 error('The matrix "M" must have 3 or 4 columns.'); 
end

M(:,3)=M(:,3)/L; %Absorbance per unit length. 

%Fit: 
sxx=sum(M(:,4).*(M(:,1).^2)); 
syy=sum(M(:,4).*(M(:,2).^2)); 
sxy=sum(M(:,1).*M(:,2).*M(:,4)); 
sxz=sum(M(:,1).*M(:,3).*M(:,4)); 
syz=sum(M(:,2).*M(:,3).*M(:,4)); 
sx=sum(M(:,1).*M(:,4));
sy=sum(M(:,2).*M(:,4)); 
sz=sum(M(:,3).*M(:,4));
sw=sum(M(:,4)); 
if bg==0 
 A=[sxx sxy; sxy syy]; 
 Z=[sxz; syz]; 
elseif bg==1
 A=[sxx sxy sx; sxy syy sy; sx sy sw]; 
 Z=[sxz; syz; sz]; 
else
 error('bg must be equal to 0 (background is subtracted) or 1 (background is not subtracted)'); 
end
if det(A)==0
 error('The two spectra must differ.'); 
end

B=inv(A); 
N=B*Z; %Concentration of both species and eventual background level. 


%Calculation of confidence intervals:
z=N(1)*M(:,1) + N(2)*M(:,2); 
if bg==1 
 z=z+N(3); 
end
S=sum(M(:,4).*((z-M(:,3)).^2)); %Sum of squares of weighted errors 

F=M(:,[1 2]);
if bg==1
 F=[F ones(n,1)]; 
end
F=(F.')*diag(M(:,4))*F; 
if bg==0
 R=inv(F)*S/(n-2);
 R=[sqrt(R(1,1)) sqrt(R(2,2))];
elseif bg==1 
 R=inv(F)*S/(n-3);
 R=[sqrt(R(1,1)) sqrt(R(2,2)) sqrt(R(3,3))]; 
end 


%Division of sum of squares between the two spectra, i.e. an estimation how the individual substances contribute to the sum of squares of errors: 

odch=M(:,4).*(z-M(:,3)).^2; 
cha=0; chb=0; 
for j=1:n, 
 if M(j,1)~=0 | M(j,2)~=0
  cha=cha + odch(j)*M(j,1)/(S*sum(M(:,1))*(M(j,1)/sum(M(:,1)) + M(j,2)/sum(M(:,2))));
  chb=chb + odch(j)*M(j,2)/(S*sum(M(:,2))*(M(j,1)/sum(M(:,1)) + M(j,2)/sum(M(:,2))));
 end
end
Ch=[cha chb]; %Relative division of the sum of squares between both substances. "cha" gives the part of "S" that is related to the first substance and "chb" gives the part of "S" that is related to the second substance. 

%Addition of the mean sum of squares and addition of the relative deviation:  
Ch=[S/n sqrt(S)/sum(abs(M(:,4).*M(:,3))); Ch];

